
/* Includes ------------------------------------------------------------------*/

/* mbed specific header files. */
#include "mbed.h"

/* Component specific header files. */
#include "L6208.h"


/* Definitions ---------------------------------------------------------------*/
#ifdef TARGET_NUCLEO_F334R8
#define VREFA_PWM_PIN D11
#define VREFB_PWM_PIN D9
#elif TARGET_NUCLEO_F302R8
#define VREFA_PWM_PIN D11
#define VREFB_PWM_PIN D15 /* HW mandatory patch: bridge manually D9 with D15 */
#else
#define VREFA_PWM_PIN D3
#define VREFB_PWM_PIN D9
#endif

/* Variables -----------------------------------------------------------------*/

/* Initialization parameters of the motor connected to the expansion board. */
l6208_init_t init =
{
  1500,            //Acceleration rate in step/s^2 or (1/16)th step/s^2 for microstep modes
  50,              //Acceleration current torque in % (from 0 to 100)
  1000,            //Deceleration rate in step/s^2 or (1/16)th step/s^2 for microstep modes
  50,              //Deceleration current torque in % (from 0 to 100)
  1500,            //Running speed in step/s or (1/16)th step/s for microstep modes
  50,              //Running current torque in % (from 0 to 100)
  30,              //Holding current torque in % (from 0 to 100)
  STEP_MODE_1_16,  //Step mode via enum motorStepMode_t
  FAST_DECAY,      //Decay mode via enum motorDecayMode_t
  0,               //Dwelling time in ms
  FALSE,           //Automatic HIZ STOP
  100000           //VREFA and VREFB PWM frequency (Hz)
};

/* Motor Control Component. */
L6208 *motor;

/* Functions -----------------------------------------------------------------*/

/**
 * @brief  This is an example of user handler for the flag interrupt.
 * @param  None
 * @retval None
 * @note   If needed, implement it, and then attach and enable it:
 *           + motor->attach_flag_irq(&my_flag_irq_handler);
 *           + motor->enable_flag_irq();
 *         To disable it:
 *           + motor->DisbleFlagIRQ();
 */
void my_flag_irq_handler(void)
{
  printf("    WARNING: \"FLAG\" interrupt triggered:\r\n");
  motor->disable();
  printf("    Motor disabled.\r\n\n");
}

/**
 * @brief  This is an example of error handler.
 * @param[in] error Number of the error
 * @retval None
 * @note   If needed, implement it, and then attach it:
 *           + motor->attach_error_handler(&my_error_handler);
 */
void my_error_handler(uint16_t error)
{
  /* Printing to the console. */
  printf("Error %d detected\r\n\n", error);
  
  /* Infinite loop */
  while (true) {
  }    
}

/* Main ----------------------------------------------------------------------*/

int main()
{

//----- Initialization 
  /* Initializing Motor Control Component. */
  motor = new L6208(D2, D8, D7, D4, D5, D6, VREFA_PWM_PIN, VREFB_PWM_PIN);
  if (motor->init(&init) != COMPONENT_OK) {
    exit(EXIT_FAILURE);
  }

  /* Attaching and enabling an interrupt handler. */
  motor->attach_flag_irq(&my_flag_irq_handler);
  motor->enable_flag_irq();
    
  /* Attaching an error handler */
  motor->attach_error_handler(&my_error_handler);
 //----- run the motor BACKWARD
  printf("--> Running the motor backward.\r\n");
  motor->run(StepperMotor::BWD);
  
  while (motor->get_status()!=STEADY) {
    /* Print reached speed to the console in step/s or microsteps/s */
    printf("    Reached Speed: %d microstep/s.\r\n", motor->get_speed());
    wait_ms(50);    
  }
  printf("    Reached Speed: %d microstep/s.\r\n", motor->get_speed());
     
  /* Wait for 1 second */  
  wait_ms(1000);
  
//----- Decrease speed while running to one quarter of the previous speed
  motor->set_max_speed(motor->get_speed()>>2);
  
  /* Wait until the motor starts decelerating */
  while (motor->get_status()==STEADY);
  /* Wait and print speed while the motor is not steady running */
  while (motor->get_status()!=STEADY) {
    /* Print reached speed to the console in step/s or microsteps/s */
    printf("    Reached Speed: %d microstep/s.\r\n", motor->get_speed());
    wait_ms(50);    
  }
  printf("    Reached Speed: %d microstep/s.\r\n", motor->get_speed());  

  /* Wait for 5 seconds */  
  wait_ms(5000);
  
 //----- Change step mode to 1/4 microstepping mode  
  motor->set_step_mode(StepperMotor::STEP_MODE_1_4);

  /* Set speed, acceleration and deceleration to scale with microstep mode */
  motor->set_max_speed(motor->get_max_speed()<<4);
  motor->set_acceleration(motor->get_acceleration()<<4);
  motor->set_deceleration(motor->get_deceleration()<<4);
  
  /* Request to go position 800 (quarter steps) */
  motor->go_to(9000);

  /* Wait for the motor ends moving */
  motor->wait_while_active();
motor->go_to(-9000);

  /* Wait for the motor ends moving */
  motor->wait_while_active();
  /* Wait for 2 seconds */  
  wait_ms(2000);
int i;
  for (i = 0; i < 4; i++) {
    /* Request device to go position -3200 */
    motor->go_to(3200);

    /* Waiting while the motor is active. */
    motor->wait_while_active();

    /* Request device to go position 3200 */
    motor->go_to(-3200);
 
    /* Waiting while the motor is active. */
    motor->wait_while_active();
  }
    wait_ms(5000);  
  motor->disable();
}


